///////////////////////////////////////////////////////GLOBAL VARIABLES

///////////////////////////////////////////////////////PRE-DEFINED

function BeastClass::onAdd(%this,%scenegraph)
{
	%this.enableUpdateCallback();
	
	%this.outClawFactor = 2.5;
	%this.ClawLimitFactor = 1;
	
	%this.InitializeClaws();
	
	%this.moveSpeed = 10;
	%this.clawOffset = 2.5;
	%this.clawX = %this.clawY = %this.clawXY = 0;
	%this.speedFactorX = %this.speedFactorY = %this.speedFactorXY = %this.moveSpeed*-0.03;
	%this.clawOnceX = %this.clawOnceY = %this.clawOnceXY1 = %this.clawOnceXY2 = false;
	
	%this.frontLegLeft  = %this.makeFrontLeg();
	%this.frontLegRight = %this.makeFrontLeg();
	%this.frontLegRight.setFlipX(true);
	
	%this.tailSegmentCount = 3;
	%this.swingBase = 0;
	
	for(%i = 0; %i < %this.tailSegmentCount; %i++)
	{
		if(%i != %this.tailSegmentCount-1)
		{
			%this.tail[%i] = %this.makeTailSegment();
			
			if(%i == 0)
				%this.tail[%i].mount(%this,0,0.6,0,false,false,false,false);
			else
			{
				%this.tail[%i].mount(%this.tail[%i-1],0,1,0,false,false,false,false);
			}
		}
		else
		{
			%this.tail[%i] = %this.makeEndTailSegment();
			%this.tail[%i].mount(%this.tail[%i-1],0,1.5,0,false,false,false,false);
		}
		
		
	}
	
	%this.speedMax = 10;
	%this.speedMin = -%this.speedMax;
	%this.hor = 0;
	%this.ver = 0;
	%this.moveRandomly(%this);
}

function BeastClass::onUpdate(%this)
{
	%this.updateClaws();
	
	%this.legPositioning();
	
	//TAIL
	for(%i = 0; %i < %this.tailSegmentCount; %i++)
	{
		if(%this.getLinearVelocityX() < 0)
			%this.swingTail(%i,"LEFT");
			
		if(%this.getLinearVelocityX() > 0)
			%this.swingTail(%i,"RIGHT");
			
		if(%this.getLinearVelocityX() == 0)
			%this.swingTail(%i,"IDLE");
	}
}

function BeastClass::onTimer(%this)
{
	%this.moveRandomly(%this);
}

function BeastClass::onRemove(%this)
{
	if(isObject(%this.frontLegLeft))
	{
		makeDeadBodyPart(%this.frontLegLeft);
		%this.frontLegLeft.safeDelete();
	}
	if(isObject(%this.frontLegRight))
	{
		makeDeadBodyPart(%this.frontLegRight);
		%this.frontLegRight.safeDelete();
	}

	
	if(isObject(%this.claw[0]))
		makeDeadBodyPart(%this.claw[0]);
	if(isObject(%this.claw[1]))
		makeDeadBodyPart(%this.claw[1]);
	%this.claw[0].safeDelete();
	%this.claw[1].safeDelete();
	
	for(%i = 0; %i < %this.tailSegmentCount; %i++)
	{
		if(isObject(%this.tail[%i]))
		{
			makeDeadBodyPart(%this.tail[%i]);
			%this.tail[%i].safeDelete();
		}
	}
	
	makeDeadBodyPart(%this);
}

//////////////////////////////////////////////////////////SUPPORT

/*
	This creates the 2 claws that the beast needs.
*/
function BeastClass::InitializeClaws(%this)
{
	for(%i = 0; %i < 2; %i++)
	{
		%this.claw[%i] = new t2dStaticSprite()
		{
			ImageMap = "BeastClawIMImageMap"; 
			Config = "BeastBaddieClaw";
			Scenegraph = %this.getScenegraph();
			Position = %this.getPosition();
			CollisionList = "0 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31";
		};
	}
	
	//Front claws.
	%this.claw[0].setPosition(%this.getPositionX()-%this.clawOffset,%this.getPositionY());
	%this.claw[1].setPosition(%this.getPositionX()+%this.clawOffset,%this.getPositionY());
}

/*
	This moves and updates the claws of the beast.
*/
function BeastClass::UpdateClaws(%this)
{	
	if(mAbs(%this.clawX) > %this.ClawLimitFactor)
		%this.speedFactorX *= -1;
		
	if(mAbs(%this.clawY) > %this.ClawLimitFactor)
		%this.speedFactorY *= -1;
		
	if(mAbs(%this.clawXY) > %this.ClawLimitFactor)
		%this.speedFactorXY *= -1;
	
	//HORIZONTAL	
	if(%this.getLinearVelocityX() != 0 && %this.getLinearVelocityY() == 0)
	{
		//This resets the claws positions once before moving them as to avoid the claws getting stuck in weird places.
		if(%this.clawOnceX == false)
		{
			//Front claws.
			%this.claw[0].setPosition(%this.getPositionX()-%this.clawOffset,%this.getPositionY());
			%this.claw[1].setPosition(%this.getPositionX()+%this.clawOffset,%this.getPositionY());
	
			%this.clawOnceX = true;
		}		
		
		//Front claws.
		%this.claw[0].setPositionX(%this.getPositionX()-%this.outClawFactor-%this.clawX);
		%this.claw[1].setPositionX(%this.getPositionX()+%this.outClawFactor-(%this.clawX*-1));
	
		%this.clawX += %this.speedFactorX;
		
		//Reset other directions variables.
		%this.clawY = 0;
		%this.clawXY = 0;
		%this.clawOnceY = false;
		%this.clawOnceXY1 = false;
		%this.clawOnceXY2 = false;
	}	
	
	//VERTICAL	
	if(%this.getLinearVelocityY() != 0 && %this.getLinearVelocityX() == 0)
	{
		if(%this.clawOnceY == false)
		{
			//Front claws.
			%this.claw[0].setPosition(%this.getPositionX()-%this.clawOffset,%this.getPositionY());
			%this.claw[1].setPosition(%this.getPositionX()+%this.clawOffset,%this.getPositionY());
	
			%this.clawOnceY = true;
		}
		
		//Front claws.
		%this.claw[0].setPositionY(%this.getPositionY()-%this.outClawFactor-%this.clawY);
		%this.claw[1].setPositionY(%this.getPositionY()-%this.outClawFactor-(%this.clawY*-1));

		%this.clawY += %this.speedFactorY;
		
		//Reset other directions variables.
		%this.clawX = 0;
		%this.clawXY = 0;
		%this.clawOnceX = false;
		%this.clawOnceXY1 = false;
		%this.clawOnceXY2 = false;
	}
	
	//UPPER LEFT TO LOWER RIGHT
	if((%this.getLinearVelocityX() < 0 && %this.getLinearVelocityY() < 0)||(%this.getLinearVelocityX() > 0 && %this.getLinearVelocityY() > 0))
	{
		if(%this.clawOnceXY1 == false)
		{
			//Front claws.
			%this.claw[0].setPosition(%this.getPositionX()+%this.clawOffset,%this.getPositionY());
			%this.claw[1].setPosition(%this.getPositionX()+%this.clawOffset,%this.getPositionY());
			
			%this.clawOnceXY1 = true;
		}
		
		//Front claws.
		%this.claw[0].setPosition(%this.getPositionX()-%this.outClawFactor-%this.clawXY,%this.getPositionY()-%this.outClawFactor-%this.clawXY);
		%this.claw[1].setPosition(%this.getPositionX()+%this.outClawFactor-(%this.clawXY*-1),%this.getPositionY()-%this.outClawFactor-(%this.clawXY*-1));

		%this.clawXY += %this.speedFactorXY;
		
		//Reset other directions variables.
		%this.clawX = 0;
		%this.clawY = 0;
		%this.clawOnceX = false;
		%this.clawOnceY = false;
		%this.clawOnceXY2 = false;
	}
	
	
	//UPPER RIGHT TO LOWER LEFT
	if((%this.getLinearVelocityX() > 0 && %this.getLinearVelocityY() < 0)||(%this.getLinearVelocityX() < 0 && %this.getLinearVelocityY() > 0))
	{
		if(%this.clawOnceXY2 == false)
		{
			//Front claws.
			%this.claw[0].setPosition(%this.getPositionX()-%this.clawOffset,%this.getPositionY());
			%this.claw[1].setPosition(%this.getPositionX()+%this.clawOffset,%this.getPositionY());
	
			%this.clawOnceXY2 = true;
		}
		
		//Front claws.
		%this.claw[0].setPosition(%this.getPositionX()-%this.outClawFactor-%this.clawXY,%this.getPositionY()-%this.outClawFactor+%this.clawXY);
		%this.claw[1].setPosition(%this.getPositionX()+%this.outClawFactor-(%this.clawXY*-1),%this.getPositionY()-%this.outClawFactor+(%this.clawXY*-1));

		%this.clawXY += %this.speedFactorXY;
		
		//Reset other directions variables.
		%this.clawX = 0;
		%this.clawY = 0;
		%this.clawOnceX = false;
		%this.clawOnceY = false;
		%this.clawOnceXY1 = false;
	}
}

/*
	This handles the legs positions at all times.
*/
function BeastClass::legPositioning(%this)
{	
	%FLLX = %this.getPositionX()-((%this.getPositionX()-%this.claw[0].getPositionX())/2);
	%FLLY = %this.getPositionY()-((%this.getPositionY()-%this.claw[0].getPositionY())/2);
	
	%FLRX = %this.getPositionX()-((%this.getPositionX()-%this.claw[1].getPositionX())/2);
	%FLRY = %this.getPositionY()-((%this.getPositionY()-%this.claw[1].getPositionY())/2);
	
	%this.frontLegLeft.setPosition(%FLLX-0.5,%FLLY);
	%this.frontLegRight.setPosition(%FLRX+0.5,%FLRY);
	
	%beastPosUpper = (%this.getPositionX())SPC(%this.getPositionY()-1);
	%beastPosLower = (%this.getPositionX())SPC(%this.getPositionY()+1);
	
	%this.frontLegLeft.setRotation(90+angleFromPositions(%beastPosUpper,%this.claw[0].getPosition()));
	%this.frontLegRight.setRotation(90+angleFromPositions(%beastPosUpper,%this.claw[1].getPosition()));
}

/*
	This makes a cat's front leg.
*/
function BeastClass::makeFrontLeg(%this)
{
	%frontLeg = new t2dStaticSprite()
	{
		ImageMap = "BeastTailSegmentIMImageMap"; 
		Config = "BeastBaddieLegSegment";
		Scenegraph = %this.getScenegraph();
		Position = %this.getPosition();
		CollisionList = "0 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31";
	};
	return %frontLeg;
}

/*
	This makes a cat tail segment swing.
	PARAMS: %tailIndex = which tail segment to swing. %method = tells the function if we are going left, right or not moving horizontally.
*/
function BeastClass::swingTail(%this,%tailIndex,%method)
{
	//This slows down the swinging so it doesn't get out of hand.
	if((%this.tail[0].getRotation() < %this.swingBase && %this.tail[0].tailArrest > %this.swingBase) || 
		(%this.tail[0].getRotation() > %this.swingBase && %this.tail[0].tailArrest < %this.swingBase))
	{
		for(%i = 0; %i < $TAILSEGMENTCOUNT; %i++)
		{
			if(%method !$= "RIGHT")
			{
			if(%this.tail[%i].swingFactor > %this.swingBase)
				%this.tail[%i].swingFactor = 7;
				
			if(%this.tail[%i].swingFactor < %this.swingBase)
				%this.tail[%i].swingFactor = -7;
			}
			
			if(%method $= "LEFT")
			{
			if(%this.tail[%i].swingFactor > %this.swingBase)
				%this.tail[%i].swingFactor = -7;
				
			if(%this.tail[%i].swingFactor < %this.swingBase)
				%this.tail[%i].swingFactor = 7;
			}
					
			%this.tail[%i].setRotation(%this.swingBase);	
		}
	}
	
	%this.tail[0].tailArrest = %this.tail[0].getRotation();
		
	if(%method $= "IDLE")
		%this.swingBase = 0;
	
	if(%method $= "LEFT")
		%this.swingBase = -30;
	
	if(%method $= "RIGHT")
		%this.swingBase = 30;
		
		
	//Move the tail segments.
	if(%tailIndex == 0)
		%this.tail[%tailIndex].setRotation(%this.tail[%tailIndex].getRotation()+%this.tail[%tailIndex].swingFactor);
	else
		%this.tail[%tailIndex].setRotation(%this.tail[%tailIndex-1].getRotation()+%this.tail[%tailIndex-1].swingFactor*-4);
	
	//Add swinging momentum.
	if(%this.tail[%tailIndex].getRotation() > %this.swingBase)
		%this.tail[%tailIndex].swingFactor -= %this.tail[%tailIndex].rotateAdd;
		
	if(%this.tail[%tailIndex].getRotation() < %this.swingBase)
		%this.tail[%tailIndex].swingFactor += %this.tail[%tailIndex].rotateAdd;
}

/*
	This makes a beast's tail segment.
*/
function BeastClass::makeTailSegment(%this)
{
	%tail = new t2dStaticSprite()
	{
		ImageMap = "BeastTailSegmentIMImageMap"; 
		Config = "BeastBaddieTailSegment";
		Scenegraph = %this.getScenegraph();
		Position = %this.getPosition();
		CollisionList = "0 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31";
	};
	%tail.swingFactor = 7;
	%tail.rotateAdd = 0.5;
	%this.tail[0].tailArrest = 1000;
	return %tail;
}

/*
	This makes a beast's tail end segment.
*/
function BeastClass::makeEndTailSegment(%this)
{
	%tail = new t2dStaticSprite()
	{
		ImageMap = "BeastTailEndSegmentIMImageMap"; 
		Config = "BeastBaddieTailEnd";
		Scenegraph = %this.getScenegraph();
		Position = %this.getPosition();
		CollisionList = "0 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31";
	};
	%tail.swingFactor = 7;
	%tail.rotateAdd = 0.5;
	return %tail;
}

/*
This handles the randomness of the monsters movements
*/
function BeastClass::moveRandomly(%this)
{
	
	%this.hor = getRandom(%this.speedMin, %this.speedMax);
	if(%this.hor > 0) 
		%this.hor = %this.speedMax;
	else if(%this.hor < 0) 
		%this.hor = %this.speedMin;
		
	%this.ver = getRandom(%this.speedMin, %this.speedMax);
	if(%this.ver > 0) 
		%this.ver = %this.speedMax;
	else if(%this.ver < 0) 
		%this.ver = %this.speedMin;
	
	%this.setLinearVelocity(%this.hor,%this.ver);
	
	%this.setTimerOn(getRandom(800,1200));
}
