///////////////////////////////////////////////////////PRE-DEFINED
function DirigibleClass::onAdd(%this, %scenegraph)
{
	if($CATEDITOR == -1) %this.safeDelete();	
	
	//Initialize
	%this.enableUpdateCallback();
	
	//Variables
	%this.cruiseSpeed = 0;
	%this.maxSpeed = 5;
	%this.pos = 0;
	%this.radius = 30;
	%this.softCircleMin = 1;
	%this.softCircleMax = 10;
	%this.softCircle = %this.softCircleMin;
	%this.animTrigger = "LEFT";
	%this.dirRotation = 90;
	%this.dirRotationMax = 15;
	%this.followCat = false;
	%this.scoreXOffset = -0.11;
	%this.scoreYOffset = 0.07;
	
	//SCORE DISPLAY
	%this.showingScore = new t2dTextObject()
   {
      Scenegraph = %this.Scenegraph;
		Config = "Score";
      Text = $SCORE;
      Font = "DS-Digital";
      TextAlign = "right";
      HideOverflow = "0";
      LineHeight = "2.5";
      FontSizes = "28";
      Layer = "1";
   };
   %this.showingScore.setBlendColor(0, 255, 0);
   %this.showingScore.mount(%this,%this.scoreXOffset,-%this.scoreYOffset,0,false,false,true,false);
   //%this.showingScore.setMountRotation(%this.dirRotation);
}

function DirigibleClass::onUpdate(%this)
{
	//For ease of reading and typing. These DO get updated every update.
	%CatX = MainCat.getPositionX();
	%CatY = MainCat.getPositionY();
	%CatToThis = distanceBetweenPoints(%this,MainCat,"BOTH");
	%CatAngleThis = angleFromPositions(MainCat,%this);
	
	//Speeds up the dirigible the farther it is away from the MainCat.
	%this.cruiseSpeed = %CatToThis/5;
	if(%this.cruiseSpeed < %this.maxSpeed)
		%this.cruiseSpeed = %this.maxSpeed;
	
	if(MainCat.getLinearVelocityX() $= "0" && MainCat.getLinearVelocityY() $= "0")
	{
		if(%CatToThis > %this.radius+5)//If the dirigible is far away from the cat.
		{
			%this.moveTo(MainCat.getPosition(),%this.cruiseSpeed);		
			%this.pos = %CatAngleThis;
		}else if((%CatToThis < %this.radius+5 && %CatToThis > %this.radius))//If the dirigible is close to the cat
		{
			//This all makes the dirigible easy into the orbiting a little more than a direct change of direction.
			%xTarget = %this.radius * mCos(mDegToRad(%this.pos + %this.softCircle)) + %CatX;
			%yTarget = %this.radius * mSin(mDegToRad(%this.pos + %this.softCircle)) + %CatY;
			%this.softCircle += 0.5;
			%this.moveTo(%xTarget,%yTarget,%this.cruiseSpeed);
			%this.pos = %CatAngleThis;
		}else if(%CatToThis < %this.radius+5 && %CatToThis > %this.radius-1)//We are close enough to the cat and should still orbit it.
		{
			%this.setPositionX((%this.radius * mCos(mDegToRad(%this.pos)) + %CatX));
			%this.setPositionY((%this.radius * mSin(mDegToRad(%this.pos)) + %CatY));
			if(%this.pos >= 360) %this.pos -= 360;
			%this.pos += %this.cruiseSpeed/10;
		}else
		{
			//This all makes the dirigible easy into the orbiting a little more than a direct change of direction.
			%xTarget = %this.radius * mCos(mDegToRad(%this.pos + %this.softCircle)) + %CatX;
			%yTarget = %this.radius * mSin(mDegToRad(%this.pos + %this.softCircle)) + %CatY;
			%this.softCircle += 0.5;
			%this.moveTo(%xTarget,%yTarget,%this.cruiseSpeed);
			%this.pos = %CatAngleThis;
		}
	}else
	{
		%this.moveTo(MainCat.getPosition(),%this.cruiseSpeed);		
		%this.pos = %CatAngleThis;
		%this.softCircle = %this.softCircleMin;
	}
	
	if(%this.softCircle > %this.softCircleMax)
		%this.softCircle = %this.softCircleMax;
	
	%this.updateImageMap();
}

//////////////////////////////////////////////////////////SUPPORT

/*
	This changes the look of the dirigible to face where it is going.
*/
function DirigibleClass::updateImageMap(%this)
{ 
	//This stuff happens when the cat is staying still.
	if(MainCat.getLinearVelocityX() $= "0" && MainCat.getLinearVelocityY() $= "0")
	{
		//Change the angle of the dirigible.
		if(%this.pos < 270 && %this.pos > 180)
		{
			%this.setRotation(%this.dirRotation);
			%this.dirRotation--;
		}else if(%this.pos > 270)
		{
			%this.setRotation(%this.dirRotation);
			%this.dirRotation++;
		}else if(%this.pos < 90)
		{
			%this.setRotation(%this.dirRotation);
			%this.dirRotation--;
		}else if(%this.pos < 180 && %this.pos > 90)
		{
			%this.setRotation(%this.dirRotation);
			%this.dirRotation++;
		}
		
		//Change the animation of the dirigible.
		if((%this.pos > 180 && %this.pos < 361) && %this.animTrigger $= "LEFT")
		{
			%this.playAnimation("DirigibleRightANIM");
			%this.animTrigger = "RIGHT";
			if(%this.pos < 270)
				%this.dirRotation = -%this.dirRotationMax;
				
			%this.scoreXOffset *= -1;
			%this.showingScore.mount(%this,%this.scoreXOffset,-%this.scoreYOffset,0,false,false,true,false);
		}
		
		if((%this.pos < 180 && %this.pos > 0) && %this.animTrigger $= "RIGHT")
		{
			%this.playAnimation("DirigibleLeftANIM");
			%this.animTrigger = "LEFT";
			if(%this.pos < 90)
				%this.dirRotation = -%this.dirRotationMax;
				
			%this.scoreXOffset *= -1;
			%this.showingScore.mount(%this,%this.scoreXOffset,-%this.scoreYOffset,0,false,false,true,false);
		}
	}else //This stuff happens when the cat is moving.
	{
		
		//Change the angle of the dirigible.
		if(%this.getLinearVelocityY() > 0 && %this.getPositionX() > MainCat.getPositionX())//Going down and at the right of the cat.
		{
			%this.setRotation(%this.dirRotation);
			%this.dirRotation--;
		}else if(%this.getLinearVelocityY() > 0 && %this.getPositionX() <= MainCat.getPositionX())//Going down and at the left of the cat.
		{
			%this.setRotation(%this.dirRotation);
			%this.dirRotation++;
		}else if(%this.getLinearVelocityY() < 0 && %this.getPositionX() > MainCat.getPositionX())//Going up and at the right of the cat.
		{
			%this.setRotation(%this.dirRotation);
			%this.dirRotation++;
		}else if(%this.getLinearVelocityY() < 0 && %this.getPositionX() <= MainCat.getPositionX())//Going up and at the left of the cat.
		{
			%this.setRotation(%this.dirRotation);
			%this.dirRotation--;
		}	
		
		//Change the animation of the dirigible.
		if(MainCat.getPositionX() < %this.getPositionX() && %this.animTrigger $= "RIGHT")
		{
			%this.playAnimation("DirigibleLeftANIM");
			%this.animTrigger = "LEFT";
			
			if(%this.getLinearVelocityY() > 0)
				%this.dirRotation = -%this.dirRotationMax;
			else
				%this.dirRotation = %this.dirRotationMax;
				
			%this.scoreXOffset *= -1;
		}else if(MainCat.getPositionX() > %this.getPositionX() && %this.animTrigger $= "LEFT")
		{
			%this.playAnimation("DirigibleRightANIM");
			%this.animTrigger = "RIGHT";
			if(%this.getLinearVelocityY() > 0)
				%this.dirRotation = %this.dirRotationMax;
			else
				%this.dirRotation = -%this.dirRotationMax;
				
			%this.scoreXOffset *= -1;
		}
	}
	
	if(%this.dirRotation < -%this.dirRotationMax)
		%this.dirRotation = -%this.dirRotationMax;
			
	if(%this.dirRotation > %this.dirRotationMax)
		%this.dirRotation = %this.dirRotationMax;
		
	%this.showingScore.setRotation(%this.dirRotation);
	%this.showingScore.mount(%this,%this.scoreXOffset,-%this.scoreYOffset,0,false,false,true,false);
}





function StaticDirigibleClass::onLevelLoaded(%this, %scenegraph)
{
	%this.scoreXOffset = -0.11;
	%this.scoreYOffset = 0.07;
	
	//SCORE DISPLAY
	%this.showingScore = new t2dTextObject()
   {
      Scenegraph = %this.Scenegraph;
		Config = "Score";
      Text = $SCORE;
      Font = "DS-Digital";
      TextAlign = "right";
      HideOverflow = "0";
      LineHeight = "2.5";
      FontSizes = "28";
      Layer = "1";
   };
   %this.showingScore.setBlendColor(0, 255, 0);
   %this.showingScore.mount(%this,%this.scoreXOffset,-%this.scoreYOffset,0,false,false,true,false);
}