/* 
    This returns the angle between two positions. 
    PARAMS: %firstPos = A vector or a reference to an object to take a position from, %secondPos = A vector or a reference to an object to take a position from 
*/  
function angleFromPositions(%firstPos, %secondPos)  
{  
    %x1 = getWord(%firstPos,0);  
    %y1 = getWord(%firstPos,1);  
      
    //If we were given reference to an object it wouldn't have two numbers.  
    if(%y1 $= "")  
    {  
        %x1 = %firstPos.getPositionX();  
        %y1 = %firstPos.getPositionY();  
    }  
      
    %x2 = getWord(%secondPos,0);  
    %y2 = getWord(%secondPos,1);  
      
    //If we were given reference to an object it wouldn't have two numbers.  
    if(%y2 $= "")  
    {  
        %x2 = %secondPos.getPositionX();  
        %y2 = %secondPos.getPositionY();  
    }  
      
    %xLen = %x1 - %x2;  
    %yLen = %y1 - %y2;  
      
    %angle = mRadToDeg(mAtan(%yLen,%xLen));  
  
    //This fixes the angle.  
    %angle += 180;        
              
    return %angle;  
}

/*
	This returns the distance from two points given to it in a straight line.
	PARAMS: %source = A vector or a reference to an object to take a position from, %target = A vector or a reference to an object to take a position from, %modifier = A string that indicates if the result should be HORIZONTAL, VERTICAL or BOTH
*/
function distanceBetweenPoints(%source,%target,%modifier)
{
	%x1 = getWord(%source,0);
	%y1 = getWord(%source,1);
	
	//If we were given reference to an object it wouldn't have two numbers.
	if(isObject(%source) && %y1 $= "")
	{
		%x1 = %source.getPositionX();
		%y1 = %source.getPositionY();
	}
	
	%x2 = getWord(%target,0);
	%y2 = getWord(%target,1);
	
	//If we were given reference to an object it wouldn't have two numbers.
	if(isObject(%target) && %y2 $= "")
	{
		%x2 = %target.getPositionX();
		%y2 = %target.getPositionY();
	}
	
	//We are actually just finding the hypotenuse of a triangle given its height and length.
	%xDis = mAbs(%x1 - %x2);
	%yDis = mAbs(%y1 - %y2);
	if(%modifier $= "BOTH")
		return mSqrt(%xDis*%xDis + %yDis*%yDis);
	if(%modifier $= "VERTICAL")
		return %yDis;
	if(%modifier $= "HORIZONTAL")
		return %xDis;
}

/*
	This returns the square of the number given to it.
	PARAMS: %factor = Number to square.
*/
function square(%factor)
{
	return %factor*%factor;
}

/*
	This returns the tile's X position given the coordinates and original Tile Layer.
	PARAMS: %source = Current Tile Layer, %ref = Vector holding one or two XY values
*/
function t2dTileLayer::getCurrentTileX(%this,%source,%ref){
	%MapX = %source.getPositionX();
	%MapTileMidX = %source.getTileCountX()/2;
	%CoordX = getWord(%ref,0);
	%TileWidth = %source.getTileSizeX();
	
	if(%CoordX < %MapTileMidX){
		%xPos = %MapTileMidX - %CoordX - 0.5;
		%xPos *= %TileWidth;
		%xPos = %MapX - %xPos;
	}
	else{
		%xPos = %CoordX - %MapTileMidX + 0.5;
		%xPos *= %TileWidth;
		%xPos = %MapX + %xPos;
	}
	
	return %xPos;
}

/*
	This returns the tile's Y position given the coordinates and original Tile Layer.
	PARAMS: %source = Current Tile Layer, %ref = Vector holding one or two XY values
*/
function t2dTileLayer::getCurrentTileY(%this,%source,%ref){
	%MapY = %source.getPositionY();
	%MapTileMidY = %source.getTileCountY()/2;
	%CoordY = getWord(%ref,1);
	if(%CoordY $= "")//In case the user only entered the Y value withou the X value.
		%CoordY = %ref;
		
	%TileHeight = %source.getTileSizeY();
	
	if(%CoordY < %MapTileMidY){
		%yPos = %MapTileMidY - %CoordY - 0.5;
		%yPos *= %TileHeight;
		%yPos = %MapY - %yPos;
	}
	else{
		%yPos = %CoordY - %MapTileMidY + 0.5;
		%yPos *= %TileHeight;
		%yPos = %MapY + %yPos;
	}
	
	return %yPos;
}